import React, { useState, useEffect } from 'react';
import './mustajaakko.css';

// Korttielementti yksittäisen pelikortin piirtämiseksi.
const Kortti = ({ kortti }) => (
  <div className="kortti">
    <div className="kortinArvo">{kortti.value}</div>
    <div className="kortinMaa">{kortti.maa}</div>
  </div>
);

const Blackjack = () => {
  // Tilamuuttujat ja muuttujat
  const [pakka, asetaPakka] = useState([]);
  const [pelaajanKasi, asetaPelaajanKasi] = useState([]);
  const [jakajanKasi, asetaJakajanKasi] = useState([]);
  const [gameState, setGameState] = useState('idle');
  const [message, setMessage] = useState('');

  useEffect(() => {
    jaaAloituskortit();
  }, []);

  // Luodaan uusi korttipakka.
  const luoPakka = () => {

    // Määritellään korttien värit ja niiden arvot.
    const maat = ['♠', '♣', '♦', '♥'];
    const values = [
      { label: 'A', value: 1 },
      { label: '2', value: 2 },
      { label: '3', value: 3 },
      { label: '4', value: 4 },
      { label: '5', value: 5 },
      { label: '6', value: 6 },
      { label: '7', value: 7 },
      { label: '8', value: 8 },
      { label: '9', value: 9 },
      { label: '10', value: 10 },
      { label: 'J', value: 10 },
      { label: 'Q', value: 10 },
      { label: 'K', value: 10 },
    ];

    const uusiPakka = [];
    for (let maa of maat) {
      for (let value of values) {
        uusiPakka.push({ maa, ...value });
      }
    }

    return uusiPakka;
  };

  // Sekoitetaan pakka Fisher-Yatesin algoritmilla.
  const sekoitaPakka = (pakka) => {
    for (let i = pakka.length - 1; i > 0; i--) {
      const j = Math.floor(Math.random() * (i + 1));
      [pakka[i], pakka[j]] = [pakka[j], pakka[i]];
    }
    return pakka;
  };

  // Jaetaan ensimmäiset kortit.
  const jaaAloituskortit = () => {
    const uusiPakka = luoPakka();
    const sekoitettuPakka = sekoitaPakka(uusiPakka);

    asetaPakka(sekoitettuPakka);
    asetaPelaajanKasi([sekoitettuPakka.pop(), sekoitettuPakka.pop()]);
    asetaJakajanKasi([sekoitettuPakka.pop(), sekoitettuPakka.pop()]);
    setGameState('pelaajanVuoro');
  };

  // Lasketaan tietyn kaden arvo laskemalla kaden korttien arvot yhteen.
  const laskeKadenArvo = (kasi) => {
    let value = kasi.reduce((total, kortti) => total + kortti.value, 0);
    return value;
  };
  
  // Kasitellaan pelaajan hittaaminen.
  const playerHit = () => {
    const uusiPakka = [...pakka];
    const uusiPelaajanKasi = [...pelaajanKasi, uusiPakka.pop()];

    asetaPakka(uusiPakka);
    asetaPelaajanKasi(uusiPelaajanKasi);

    const pelaajanArvo = laskeKadenArvo(uusiPelaajanKasi);
    if (pelaajanArvo > 21) {
      setGameState('peliOhi');
      setMessage('Pahoittelut, hävisit!');
    }
  };

  // Kasitellaan pelaajan standaaminen.
  const playerStand = () => {
    setGameState('jakajanVuoro');
    jakajanVuoro();
  };

  // Simuloidaan jakajan vuoroa yksinkertaisesti.
  const jakajanVuoro = () => {
    const jakajanArvo = laskeKadenArvo(jakajanKasi);
    if (jakajanArvo < 17) {
      const uusiPakka = [...pakka];
      const uusiJakajanKasi = [...jakajanKasi, uusiPakka.pop()];
  
      asetaPakka(uusiPakka);
      asetaJakajanKasi(uusiJakajanKasi);
  
      const uusiJakajanArvo = laskeKadenArvo(uusiJakajanKasi);
      if (uusiJakajanArvo > 21) {
        setGameState('peliOhi');
        setMessage('Sinä voitit! Jakaja otti liikaa');
      } else if (uusiJakajanArvo >= 17) {
        tarkistaVoittaja(uusiJakajanArvo);
      } else {
        tarkistaVoittaja(jakajanArvo);
      }
    } else {
      tarkistaVoittaja(jakajanArvo);
    }
  };

  // Maaritetaan voittaja kasien arvojen perusteella.
  const tarkistaVoittaja = (jakajanArvo) => {
    const pelaajanArvo = laskeKadenArvo(pelaajanKasi);
    if (jakajanArvo > pelaajanArvo) {
      setGameState('peliOhi');
      setMessage('Jakaja voittaa!');
    } else if (jakajanArvo < pelaajanArvo) {
      setGameState('peliOhi');
      setMessage('Sinä voitit!');
    } else {
      setGameState('peliOhi');
      setMessage("Tasapeli!");
    }
  };

  // Piirretaan hittaamisen ja standaamisen nappulat.
  const piirraNapit = () => {
    if (gameState === 'pelaajanVuoro') {
      return (
        <div className="button-group">
          <button onClick={playerHit}>Hit</button>
          <button onClick={playerStand}>Stand</button>
        </div>
      );
    }
    return null;
  };

  // Piirretään blackjack elementti jossa se peli on.
  return (
    <div className="blackjack">
      <h2>Mustajaakko</h2>
      <button onClick={jaaAloituskortit} disabled={gameState !== 'idle'}>
        Jaa
      </button>

      <div className="pelaajankasi">
        <h3>Pelaajan Käsi</h3>
        {pelaajanKasi.map((kortti, index) => (
          <Kortti key={index} kortti={kortti} />
        ))}
        <div className="kadenarvo">Arvo: {laskeKadenArvo(pelaajanKasi)}</div>
      </div>

      <div className="jakajankasi">
        <h3>Jakajan Käsi</h3>
        {jakajanKasi.map((kortti, index) => (
          <Kortti key={index} kortti={kortti} />
        ))}
        <div className="kadenarvo">Arvo: {laskeKadenArvo(jakajanKasi)}</div>
      </div>

      {piirraNapit()}

      {gameState === 'peliOhi' && (
        <div className="message">
          <p>{message}</p>
          <button onClick={jaaAloituskortit}>Pelaa Uudelleen</button>
        </div>
      )}
    </div>
  );
};

export default Blackjack;
